/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  // These should be direct imports from language/theme files
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const initHighlighter = async () => {
      try {
        log('Initializing highlighter with:', {
          langs: initialLanguages.map(l => l.name),
          themes: initialThemes.map(t => t.name),
        })

        const highlighter = await createHighlighterCore({
          // Pass langs and themes directly
          langs: initialLanguages,
          themes: initialThemes,
          // Use JavaScript regex engine which is more portable
          engine: createJavaScriptRegexEngine({
            forgiving: true,
            cache: new Map(),
          }),
        })

        setHighlighter(highlighter)
        setIsReady(true)

        log('Highlighter ready with:', {
          languages: highlighter.getLoadedLanguages(),
          themes: highlighter.getLoadedThemes(),
        })
      }
      catch (err) {
        console.error('Failed to initialize highlighter:', err)
        setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'))
      }
    }

    initHighlighter()
  }, []) // Empty deps since inputs should be stable

  const value = useMemo(
    () => ({
      highlighter,
      isReady,
      error,
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
